from Subtitle import *

s = 100

def getNoneRepeatRand(range, repeat):
    num = random.randint(0, range)
    while num == repeat:
        num = random.randint(0, range)
    return num

class Cup():
    def __init__(self, x, y):
        self.pos = [x,y]
        self.haveBalls = False
        self.show = False
        self.speed = [0,0]
        self.goal = (x,y)
        self.showtxt = load("cupShow")
        self.hidetxt = load("cupHide")
        if self.haveBalls:
            self.ball = Ball(x+75,y+140)
        else:
            self.ball = False

    def draw(self):

        if self.ball:
            self.ball.draw()

        self.pos[0] += self.speed[0]
        self.pos[1] += self.speed[1]

        if self.speed[0] > 0:
            if self.pos[0] > self.goal[0]:
                self.speed[0] = 0
                self.pos[0] = self.goal[0]
        if self.speed[0] < 0:
            if self.pos[0] < self.goal[0]:
                self.speed[0] = 0
                self.pos[0] = self.goal[0]
        if self.speed[1] > 0:
            if self.pos[1] > self.goal[1]:
                self.speed[1] = 0
                self.pos[1] = self.goal[1]
        if self.speed[1] < 0:
            if self.pos[1] < self.goal[1]:
                self.speed[1] = 0
                self.pos[1] = self.goal[1]

        if self.show:
            display.blit(self.showtxt, self.pos)
        else:
            display.blit(self.hidetxt, self.pos)

        if self.goal[0] == self.pos[0]:
            if self.goal[1] == self.pos[1]:
                return True
        return False

    def goto(self, pos, freq=20.0):
        self.goal = pos
        self.speed[0] = (pos[0]-self.pos[0])/freq
        self.speed[1] = (pos[1]-self.pos[1])/freq

    def genBall(self):
        if self.haveBalls:
            self.ball = Ball(self.pos[0]+75, self.pos[1]+140)

class Ball():
    def __init__(self,x,y):
        self.pos = [x,y]
        self.speed = 1
        self.initY = y

    def draw(self):
        if self.pos[1] < self.initY+200:
            self.pos[1] += self.speed
            self.speed += 0.5
        else:
            self.speed = 7-self.speed
            if self.speed > 0:
                self.speed = 0
            else:
                self.pos[1] += self.speed - 7
        pygame.draw.circle(display, (100,100,100), (int(self.pos[0]), int(self.pos[1])), 20, 5)

class GameSwitchCups():
    def __init__(self, crazed = False):
        self.bg = load('bgcups')
        self.cups = []
        self.crazed = crazed
        self.ended = False
        self.countDown = 100
        if not crazed:
            for i in range(4):
                self.cups.append(Cup(i*180 + 60, 170))
        else:
            for i in range(10):
                self.cups.append(Cup((i-int(i/5)*5)*163, 150*int(i/5)))
        self.cups[0].haveBalls = True
        self.queue = []
        self.showAll()
        self.started = False
        self.done = False
        self.selected = False
        self.nextstage = 1

    def draw(self):
        display.blit(self.bg, (0,0))
        flag = True
        for cup in self.cups:
            if not cup.draw():
                flag = False
        if flag:
            if self.queue:
                self.switch(self.queue[-1][0], self.queue[-1][1])
                self.queue.pop()
            else:
                if not self.done:
                    if self.started:
                        self.done = True

        if self.done:
            if not mouse_pressed[0] and mouse_changed[0]:
                for cup in self.cups:
                    if cup.haveBalls:
                        if pygame.Rect(cup.pos[0], cup.pos[1], 150, 150).collidepoint(mouse_pos[0], mouse_pos[1]):
                            game_stat[1] += 1
                            subtitle.addToQueue(("I see", "This is not a challenge for you.",))
                self.showAll()
                self.selected = True

    def showAll(self):
        for cup in self.cups:
            cup.show = True
            cup.genBall()

    def hideAll(self):
        for cup in self.cups:
            cup.show = False
            cup.ball = False
        self.genQueue()

    def update(self):
        if self.started == False:
            if self.countDown > 0:
                self.countDown -= 1
            else:
                self.hideAll()
                self.started = True

        elif self.selected:
            self.countDown += 1
            if self.countDown > 100:
                if game_stat[1] == 0 or game_stat[1] == 2:
                    subtitle.addToQueue(("You have done poorly",))
                    subtitle.addToQueue(("Next time",))
                self.ended = True

    def switch(self, cup1, cup2):
        self.cups[cup1].goto(self.cups[cup2].pos[:], freq=20-self.crazed*15)
        self.cups[cup2].goto(self.cups[cup1].pos[:], freq=20-self.crazed*15)

    def genQueue(self):
        for i in range(10+15*self.crazed):
            self.queue.append((random.randint(0,3+self.crazed*5), random.randint(0,3+self.crazed*5)))

class Equation():
    def __init__(self, crazed = False):
        self.coefficients = [0,0]
        self.answer = 0
        self.crazed = crazed
        self.txt = '0+0=?'
        self.choices = [0,0,0,0,0]
        if self.crazed:
            self.coefficients = [0,0,0,0]

    def gen(self):
        if self.crazed:
            self.answer = random.randint(10,30)
            self.coefficients[0] = random.randint(1, 20)
            self.coefficients[1] = random.randint(1, 20)
            self.coefficients[2] = random.randint(10, 30)
            self.coefficients[3] = self.coefficients[0]*self.answer+self.coefficients[1]-self.coefficients[2]*self.answer
            self.txt = '{0}x+{1}={2}x+{3}'.format(self.coefficients[0], self.coefficients[1],self.coefficients[2], self.coefficients[3])
        else:
            self.coefficients[0] = random.randint(10,30)
            self.coefficients[1] = random.randint(10,30)
            self.answer = self.coefficients[0] + self.coefficients[1]
            self.txt = '{0} + {1} = ?'.format(self.coefficients[0],  self.coefficients[1])
        self.choices[0] = self.answer
        for i in range(4):
            self.choices[i + 1] = self.answer - 10 + getNoneRepeatRand(20, 10)
        random.shuffle(self.choices)

class GameSolveEquation():
    def __init__(self, crazed = False):
        self.cups = []
        self.bg = load("bgcalc")
        self.crazed = crazed
        self.ended = False
        self.equation = Equation(self.crazed)
        self.font = pygame.font.Font("fonts/FFF_Tusj.ttf", 60)
        self.fontS = pygame.font.Font("fonts/FFF_Tusj.ttf", 70)
        self.fontSM = pygame.font.Font("fonts/FFF_Tusj.ttf", 30)
        self.new()
        self.score = 0
        self.nextstage = 1
        self.chances = 3

    def update(self):
        pass

    def draw(self):
        display.blit(self.bg, (0,0))
        if self.equation.answer:
            display.blit(self.fontSM.render("question:"+str(self.score)+"/10", 0, (80, 80, 80)), (0,0))
            display.blit(self.fontSM.render("chances:" + str(self.chances), 0, (80, 80, 80)), (0, 30))
            display.blit(self.font.render(self.equation.txt, 0, (210,210,210)), self.getDisplayPosByString(self.equation.txt))
            for i in range(5):
                txt = str(self.equation.choices[i])
                tw, th = self.fontS.size(txt)
                if pygame.Rect(i*140+80, 420, tw, th).collidepoint(mouse_pos[0], mouse_pos[1]):
                    display.blit(self.fontS.render(txt, 0, (210, 210, 210)), (i * 140 + 80, 420))
                    if not mouse_pressed[0] and mouse_changed[0]:
                        if self.equation.answer == self.equation.choices[i]:
                            self.score += 1
                            if self.score >= 10:
                                game_stat[3] += 1
                                self.ended = True
                                subtitle.addToQueue(("Impressive",))
                        else:
                            self.chances -= 1
                        self.new()
                else:
                    display.blit(self.fontS.render(txt, 0, (180, 180, 180)), (i * 140 + 80, 420))
            if self.chances <= 0:
                self.ended = True
                subtitle.addToQueue(("You are out of chances",))
                subtitle.addToQueue(("Next time",))

    def new(self):
        self.equation.gen()

    def getDisplayPosByString(self, string):
        text_width, text_height = self.font.size(string)
        return (screen_width/2 - text_width / 2, screen_height/2 - 70 - text_height / 2 )

class Probe():
    def __init__(self, x, min, max, startY=120):
        self.speed = 7
        self.fronttxt = load("droneF")
        self.backtxt = load("droneB")
        self.max = max
        self.min = min
        self.y = startY
        self.x = x

    def draw(self):
        flag = False
        self.y += self.speed
        if self.y > self.max:
            self.y = self.max
            self.speed = -self.speed
            flag = True
        elif self.y < self.min:
            self.y = self.min
            self.speed = -self.speed
        if self.speed > 0:
            display.blit(self.fronttxt, (self.x, self.y))
        else:
            display.blit(self.backtxt, (self.x, self.y))
        return flag

class GameManagement():
    def __init__(self, crazed = False):
        self.ended = False
        self.crazed = crazed
        self.fontS = pygame.font.Font("fonts/FFF_Tusj.ttf", 40)
        self.ovens = [50]
        self.drones = [Probe(340, 120, 280)]
        self.oventxt = load("pipe")
        self.crystaltxt = load("crystal")
        self.nextstage = 1
        self.countDown = 500
        self.hp = 100
        self.bg = load("bgecon")
        if crazed:
            self.ovens = [50,50,50]
            self.drones = [Probe(340, 120, 280, random.randint(120,300)),
                           Probe(140, 120, 280, random.randint(120,300)),
                           Probe(540, 120, 280, random.randint(120,300)),]

    def reduce(self, oven = 0):
        self.ovens[oven] -= 15
        if self.ovens[oven] < 0:
            self.ovens[oven] = 0

    def increase(self, oven = 0):
        self.ovens[oven] += random.randint(4+self.crazed*6,10+self.crazed*5)
        if self.ovens[oven] > 100:
            self.ovens[oven] = 100

    def draw(self):
        display.blit(self.bg, (0,0))
        display.blit(self.fontS.render(str(self.countDown/30), 0, (190, 190, 190)), (375,0))
        display.blit(self.fontS.render('hp:'+str(self.hp), 0, (100, 100, 100)), (320, 550))
        if self.crazed:
            for i in range(3):
                display.blit(self.crystaltxt, (145+i*200, 100))
            for i in range(3):
                if self.drones[i].draw():
                    self.increase(i)
            tw, th = self.fontS.size("reduce")
            for i in range(3):
                if pygame.Rect(125+i*200, 400, tw, th).collidepoint(mouse_pos[0], mouse_pos[1]):
                    if not mouse_pressed[0] and mouse_changed[0]:
                        self.reduce(i)
                    display.blit(self.fontS.render("reduce", 0, (185, 185, 185)), (125+i*200, 400))
                else:
                    display.blit(self.fontS.render("reduce", 0, (250, 250, 250)), (125+i*200, 400))
                if self.ovens[i] > 75:
                    display.blit(self.fontS.render("too many!", 0, (20, 20, 20)), (110 + i * 200, 480))
                    self.hp -= 1
                elif self.ovens[i] < 25:
                    display.blit(self.fontS.render("too little!", 0, (24, 24, 24)), (120 + i * 200, 480))
                    self.hp -= 1
                display.blit(self.fontS.render(str(self.ovens[i]), 0, (100, 100, 100)), (160 + i * 200, 440))
                display.blit(self.oventxt, (120+i*200, 325))

        else:
            display.blit(self.crystaltxt, (345, 100))
            if self.drones[0].draw():
                self.increase()
            tw, th = self.fontS.size("reduce")
            if pygame.Rect(325, 400, tw, th).collidepoint(mouse_pos[0], mouse_pos[1]):
                if not mouse_pressed[0] and mouse_changed[0]:
                    self.reduce()
                display.blit(self.fontS.render("reduce", 0, (185,185,185)), (325,400))
            else:
                display.blit(self.fontS.render("reduce", 0, (250,250,250)), (325,400))
            if self.ovens[0] > 75:
                display.blit(self.fontS.render("too many!", 0, (20, 20, 20)), (310, 480))
                self.hp -= 1
            elif self.ovens[0] < 25:
                display.blit(self.fontS.render("too little!", 0, (24, 24, 24)), (320, 480))
                self.hp -= 1
            display.blit(self.fontS.render(str(self.ovens[0]), 0, (100, 100, 100)), (360, 440))
            display.blit(self.oventxt, (320, 325))

    def update(self):
        self.countDown -= 1
        if self.countDown <= 0:
            self.ended = True
            game_stat[2] += 1
            subtitle.addToQueue(("Do you play Starcraft?",))

        if self.hp <= 0:
            self.ended = True
            subtitle.addToQueue(("You ran out of hp",))
            subtitle.addToQueue(("Next time",))

class Panel():
    path0 = load("path")
    path1 = pygame.transform.rotate(path0, 90)
    path2 = pygame.transform.rotate(path1, 90)
    path3 = pygame.transform.rotate(path2, 90)
    paths = (path0, path1, path2, path3, path0, path1, path2, path3)
    select = load("select")

    def __init__(self, x, y, connect):
        self.pos = (x,y)
        self.connect = connect
        self.rotate = random.randint(0,3)

    def turn(self):
        self.rotate += 1
        if self.rotate > 3:
            self.rotate = 0

    def draw(self, offset=(0,0)):
        if pygame.Rect(self.pos[0]+offset[0], self.pos[1]+offset[1], s, s).collidepoint(mouse_pos[0], mouse_pos[1]):
            display.blit(self.select, (self.pos[0]+offset[0], self.pos[1]+offset[1]))
            if mouse_changed[0] and not mouse_pressed[0]:
                self.turn()
        display.blit(self.paths[self.connect[0]+self.rotate], (self.pos[0]+offset[0], self.pos[1]+offset[1]))
        display.blit(self.paths[self.connect[1]+self.rotate], (self.pos[0]+offset[0], self.pos[1]+offset[1]))

class GameConnect():
    def __init__(self, crazed = False):
        self.ended = False
        self.bg = load("bgconnect")
        self.panels = [Panel(0, 0, (1, 3)) ,Panel(0, s, (2, 3)), Panel(0, 2*s, (0, 2)), Panel(0, 3*s, (0, 1)),
                       Panel(s, 0, (0, 1)), Panel(s, s, (1, 3)), Panel(s, 2 * s, (0, 2)), Panel(s, 3 * s, (1, 3)),
                       Panel(2*s, 0, (0, 3)), Panel(2*s, s, (1, 3)), Panel(2*s, 2 * s, (0, 3)), Panel(2*s, 3 * s, (2, 3)),
                       Panel(3*s, 0, (3, 2)), Panel(3*s, s, (0, 1)), Panel(3*s, 2 * s, (1, 3)), Panel(3*s, 3 * s, (0, 2)),]
        self.nextstage = 1
        self.crazed = crazed

    def draw(self):
        display.blit(self.bg,(0,0))
        for i in range(len(self.panels)):
            if self.crazed:
                if random.randint(0,150) == 0:
                    self.panels[i].turn()
            self.panels[i].draw((200,100))

    def check(self):
        if self.panels[1].rotate or self.panels[3].rotate or self.panels[12].rotate or self.panels[13].rotate:
            return False
        if self.panels[2].rotate == 1 or self.panels[2].rotate == 3:
            return False
        if self.panels[5].rotate == 1 or self.panels[5].rotate == 3:
            return False
        if self.panels[9].rotate == 1 or self.panels[9].rotate == 3:
            return False
        return True

    def update(self):
        if self.check():
            self.ended = True
            game_stat[0] += 1
            subtitle.addToQueue(("Could not see how you can lose",))
            subtitle.addToQueue(("Anyways, nice run",))

class GettableObjects():
    pass

class GameRoomEscape():
    def __init__(self,crazed=False):
        self.ended = True
        self.nextstage = 1

    def draw(self):
        pass

    def update(self):
        pass
